import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

class ITabBar extends StatefulWidget {
  const ITabBar({
    Key key,
    this.tabs,
    this.hasAppBar = false,
    this.hasBottomAppBar = false,
    this.appBarHeight = 0,
    this.bottomAppBarHeight = 0,
    this.margin,
    this.tabChanged,
    this.indexDefault = 0,
    this.labelColor,
    this.unselectedLabelColor,
    this.indicatorColor,
  }) : super(key: key);

  final List<ITab> tabs;
  final bool hasAppBar;
  final bool hasBottomAppBar;
  final double appBarHeight;
  final double bottomAppBarHeight;
  final EdgeInsetsGeometry margin;
  final Function tabChanged;
  final int indexDefault;
  final Color labelColor;
  final Color unselectedLabelColor;
  final Color indicatorColor;

  @override
  _ITabBarState createState() => _ITabBarState();
}

class _ITabBarState extends State<ITabBar> with SingleTickerProviderStateMixin {
  TabController _tabController;
  List<Widget> _tabContents;
  List<Widget> _tabIconsSelected;
  List<Widget> _tabIconsUnSelected;
  List<Widget> _tabIconsContent;
  int _index;

  @override
  void initState() {
    super.initState();
    _index = widget.indexDefault;

    _tabController = TabController(
        vsync: this, length: widget.tabs.length, initialIndex: _index);
    _tabController.addListener(() {
      changedTab(_tabController.index);
      widget.tabChanged(_tabController.index);
    });

    _tabContents = widget.tabs.map((iTab) => iTab.content).toList();
    _tabIconsSelected = widget.tabs.map((iTab) => iTab.iconSelected).toList();
    _tabIconsUnSelected =
        widget.tabs.map((iTab) => iTab.iconUnSelected).toList();

    _tabIconsContent = widget.tabs.map((iTab) => iTab.iconUnSelected).toList();
    _tabIconsContent[_index] = _tabIconsSelected[_index];
  }

  @override
  void dispose() {
    _tabController.dispose();
    super.dispose();
  }

  void changedTab(int index) {
    setState(() {
      if (_index != index) {
        _tabIconsContent[_index] = _tabIconsUnSelected[_index];
        _index = index;
        _tabIconsContent[_index] = _tabIconsSelected[_index];
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    var _appBar;
    if (widget.hasAppBar) {
      _appBar = PreferredSize(
          preferredSize: Size.fromHeight(widget.appBarHeight),
          child: Container(
              margin: widget.margin,
              color: Color(0x00000000),
              child: AppBar(
                  bottom: TabBar(
                      tabs: _tabIconsContent,
                      controller: _tabController,
                      indicatorColor: widget.indicatorColor,
                      unselectedLabelColor: widget.unselectedLabelColor,
                      labelColor: widget.labelColor))));
    }

    var _bottomAppBar;
    if (widget.hasBottomAppBar) {
      _bottomAppBar = BottomAppBar(
          child: Container(
              height: widget.bottomAppBarHeight,
              margin: widget.margin,
              color: Color(0x00000000),
              child: TabBar(
                  tabs: _tabIconsContent,
                  controller: _tabController,
                  indicatorColor: widget.indicatorColor,
                  unselectedLabelColor: widget.unselectedLabelColor,
                  labelColor: widget.labelColor)));
    }

    return Scaffold(
        appBar: _appBar,
        bottomNavigationBar: _bottomAppBar,
        body: TabBarView(children: _tabContents, controller: _tabController));
  }
}

class ITab {
  final Widget content;
  final Widget iconUnSelected;
  final Widget iconSelected;

  const ITab({this.content, this.iconUnSelected, this.iconSelected});
}
